@@ -10,41 +10,43 @@ from camunda.utils.utils import get_exception_detail
 
 
 class ExternalTaskWorker:
-    worker_id = 0
     DEFAULT_SLEEP_SECONDS = 300
 
-    def __init__(self, base_url=ENGINE_LOCAL_BASE_URL, config=frozendict({})):
-        self.worker_id = str(type(self).worker_id)
-        type(self).worker_id += 1
+    def __init__(self, worker_id, base_url=ENGINE_LOCAL_BASE_URL, config=frozendict({})):
+        self.worker_id = worker_id
         self.client = ExternalTaskClient(self.worker_id, base_url, config)
         self.executor = ExternalTaskExecutor(self.worker_id, self.client)
         self.config = config
         self._log_with_context("Created new External Task Worker")
 
-    def subscribe(self, topic_names, action):
+    def subscribe(self, topic_names, action, process_variables=None):
         while True:
-            self.fetch_and_execute(topic_names, action)
+            self._fetch_and_execute_safe(topic_names, action, process_variables)
 
         self._log_with_context("Stopping worker")
 
-    def fetch_and_execute(self, topic_names, action):
-        response = self._fetch_and_lock(topic_names)
-        tasks = self._parse_response(response, topic_names)
-        self._execute_tasks(tasks, action)
-
-    def _fetch_and_lock(self, topic_names):
+    def _fetch_and_execute_safe(self, topic_names, action, process_variables=None):
         try:
-            self._log_with_context(f"Fetching and Locking external tasks for Topics: {topic_names}")
-            return self.client.fetch_and_lock(topic_names)
+            self.fetch_and_execute(topic_names, action, process_variables)
         except Exception as e:
             sleep_seconds = self._get_sleep_seconds()
-            self._log_with_context(f'error fetching and locking tasks: {get_exception_detail(e)} '
-                                   f'for topic(s)={topic_names}. retrying after {sleep_seconds} seconds', exc_info=True)
+            self._log_with_context(f'error fetching and executing tasks: {get_exception_detail(e)} '
+                                   f'for topic(s)={topic_names} with Process variables: {process_variables}. '
+                                   f'retrying after {sleep_seconds} seconds', exc_info=True)
             time.sleep(sleep_seconds)
 
-    def _parse_response(self, response, topic_names):
+    def fetch_and_execute(self, topic_names, action, process_variables=None):
+        resp_json = self._fetch_and_lock(topic_names, process_variables)
+        tasks = self._parse_response(resp_json, topic_names)
+        self._execute_tasks(tasks, action)
+
+    def _fetch_and_lock(self, topic_names, process_variables=None):
+        self._log_with_context(f"Fetching and Locking external tasks for Topics: {topic_names} "
+                               f"with Process variables: {process_variables}")
+        return self.client.fetch_and_lock(topic_names, process_variables)
+
+    def _parse_response(self, resp_json, topic_names):
         tasks = []
-        resp_json = response and response.json()
         if resp_json:
             for context in resp_json:
                 task = ExternalTask(context)
@@ -67,7 +69,7 @@ class ExternalTaskWorker:
                                    log_level='error', exc_info=True)
 
     def _log_with_context(self, msg, topic=None, task_id=None, log_level='info', **kwargs):
-        context = frozendict({"WORKER_ID": self.worker_id, "TOPIC": topic, "TASK_ID": task_id})
+        context = frozendict({"WORKER_ID": str(self.worker_id), "TOPIC": topic, "TASK_ID": task_id})
         log_with_context(msg, context=context, log_level=log_level, **kwargs)
 
     def _get_sleep_seconds(self):
